Index: inundation/ga/storm_surge/pyvolution/general_mesh.py
===================================================================
--- inundation/ga/storm_surge/pyvolution/general_mesh.py	(revision 714)
+++ inundation/ga/storm_surge/pyvolution/general_mesh.py	(revision 715)
@@ -195,8 +195,17 @@
         
         if (indexes ==  None):
-            indexes = range(len(self))
+            indexes = range(len(self))  #len(self)=number of elements
             
         return  take(self.triangles, indexes)
-    
+
+    def get_unique_vertices(self,  indexes=None):
+        triangles = self.get_vertices(indexes=indexes)
+        unique_verts = {}
+        for triangle in triangles:
+           unique_verts[triangle[0]] = 0 
+           unique_verts[triangle[1]] = 0 
+           unique_verts[triangle[2]] = 0
+        return unique_verts.keys()   
+        
     def build_vertexlist(self):
         """Build vertexlist index by vertex ids and for each entry (point id)
Index: inundation/ga/storm_surge/pyvolution/region.py
===================================================================
--- inundation/ga/storm_surge/pyvolution/region.py	(revision 714)
+++ inundation/ga/storm_surge/pyvolution/region.py	(revision 715)
@@ -12,6 +12,6 @@
     """   
 
-    def __init__(self):
-        pass
+    def __init__(self, location='vertices'):
+        self.location = location
         
     def __call__(self, tag, elements, domain):
@@ -20,5 +20,14 @@
 
 
-class Set_Region(Region):
+    def build_indexes(self, elements, domain):
+        """
+        Return a list of triangle_id or vertex_id, depending on the location
+        """
+        if self.location == 'unique vertices':
+            return domain.get_unique_vertices(elements)
+        else:
+            return elements
+                
+class Set_region(Region):
     
     def __init__(self, tag, quantity, X, location='vertices'):
@@ -43,18 +52,26 @@
         """    
         if tag == self.tag:
-            domain.set_quantity(self.quantity, self.X, indexes = elements)
+            domain.set_quantity(self.quantity,
+                                self.X,
+                                location=self.location,
+                                indexes=self.build_indexes(elements, domain))
 
-
-class Add_Value_To_Region(Region):
+        
+class Add_value_to_region(Region):
     """
     Will add a value to the current quantity value.
     """
     
-    def __init__(self, tag, quantity, X, location='vertex'):
-        Region.__init__(self)
+    def __init__(self, tag, quantity, X, location='vertices', initial_quantity=None):
+        #I have to get this going!
+        #Region.__init__(self)
         self.tag = tag 
-        self.quantity = quantity
+        self.quantity_answer = quantity
         self.location = location
         self.X = X
+        if initial_quantity is None:
+            self.quantity_initial_value = quantity
+        else:
+            self.quantity_initial_value = initial_quantity
         if callable(X):
             raise 'This class does not work with functions'  
@@ -67,5 +84,19 @@
         """    
         if tag == self.tag:
-            new_values = domain.get_quantity(self.quantity,
-                                             indexes = elements) + self.X
-            domain.set_quantity(self.quantity, new_values, indexes = elements)
+            new_values = domain.get_quantity(self.quantity_initial_value,
+                          indexes=self.build_indexes(elements, domain),
+                          location=self.location) + self.X
+            domain.set_quantity(self.quantity_answer, new_values,
+                                indexes=self.build_indexes(elements, domain),
+                                location=self.location)
+
+
+class Stage_no_less_than_elevation(Region):
+    """
+    Will set the stage to not be less than the elevation.
+    This would be good, but it's not region dependent.
+    Wait for it to become a default for pyvolution.
+    """
+    
+    def __init__(self):
+        pass
Index: inundation/ga/storm_surge/pyvolution/shallow_water.py
===================================================================
--- inundation/ga/storm_surge/pyvolution/shallow_water.py	(revision 714)
+++ inundation/ga/storm_surge/pyvolution/shallow_water.py	(revision 715)
@@ -18,4 +18,5 @@
 
 from domain import *
+from region import *
 Generic_domain = Domain #Rename
 
Index: inundation/ga/storm_surge/pyvolution/test_general_mesh.py
===================================================================
--- inundation/ga/storm_surge/pyvolution/test_general_mesh.py	(revision 714)
+++ inundation/ga/storm_surge/pyvolution/test_general_mesh.py	(revision 715)
@@ -17,7 +17,7 @@
         pass
 
-    def test_getting_some_vertex_values(self):
+    def test_get_vertex_values(self):
         """
-        set values based on triangle lists.
+        get connectivity based on triangle lists.
         """
         from mesh_factory import rectangular
@@ -28,6 +28,5 @@
         points, vertices, boundary = rectangular(1, 3)
         domain = Domain(points, vertices, boundary)
-        quantity = Quantity(domain,[[1,1,1],[2,2,2],[3,3,3],
-                                    [4,4,4],[5,5,5],[6,6,6]])
+        
         value = [7]
         indexes = [1]
@@ -36,4 +35,25 @@
                                               domain.triangles[4]]
 
+    def test_get_unique_vertex_values(self):
+        """
+        get unique_vertex based on triangle lists.
+        """
+        from mesh_factory import rectangular
+        from shallow_water import Domain
+        from Numeric import zeros, Float
+        
+        #Create basic mesh
+        points, vertices, boundary = rectangular(1, 3)
+        domain = Domain(points, vertices, boundary)
+        
+        assert  domain.get_unique_vertices() == [0,1,2,3,4,5,6,7]
+        unique_vertices = domain.get_unique_vertices([0,1,4])
+        unique_vertices.sort()
+        assert unique_vertices == [0,1,2,4,5,6,7]
+        
+        unique_vertices = domain.get_unique_vertices([0,4])
+        unique_vertices.sort()
+        assert unique_vertices == [0,2,4,5,6,7]
+        
 #-------------------------------------------------------------
 if __name__ == "__main__":
Index: inundation/ga/storm_surge/pyvolution/test_quantity.py
===================================================================
--- inundation/ga/storm_surge/pyvolution/test_quantity.py	(revision 714)
+++ inundation/ga/storm_surge/pyvolution/test_quantity.py	(revision 715)
@@ -195,21 +195,15 @@
         quantity.set_values(f, location = 'centroids')
         assert allclose(quantity.centroid_values,
-                        [4.0/3, 8.0/3, 10.0/3, 10.0/3])        
-
-
+                        [4.0/3, 8.0/3, 10.0/3, 10.0/3])
 
 
     def test_set_vertex_values(self):
         quantity = Quantity(self.mesh4)
-
-
         quantity.set_vertex_values([0,1,2,3,4,5])
-
 
         assert allclose(quantity.vertex_values,
                         [[1,0,2], [1,2,4], [4,2,5], [3,1,4]])
-        
-        assert allclose(quantity.centroid_values, [1., 7./3, 11./3, 8./3]) #Centroid
-        
+        assert allclose(quantity.centroid_values,
+                        [1., 7./3, 11./3, 8./3]) #Centroid
         assert allclose(quantity.edge_values, [[1., 1.5, 0.5],
                                                [3., 2.5, 1.5],
@@ -218,5 +212,11 @@
 
 
-
+    def test_set_vertex_values_subset(self):
+        quantity = Quantity(self.mesh4)
+        quantity.set_vertex_values([0,1,2,3,4,5])
+        quantity.set_vertex_values([0,20,30,50], indexes = [0,2,3,5])
+
+        assert allclose(quantity.vertex_values,
+                        [[1,0,20], [1,20,4], [4,20,50], [30,1,4]])
 
     def test_set_vertex_values_using_general_interface(self):
@@ -229,6 +229,7 @@
         assert allclose(quantity.vertex_values,
                         [[1,0,2], [1,2,4], [4,2,5], [3,1,4]])
-        
-        assert allclose(quantity.centroid_values, [1., 7./3, 11./3, 8./3]) #Centroid
+
+        #Centroid
+        assert allclose(quantity.centroid_values, [1., 7./3, 11./3, 8./3])
         
         assert allclose(quantity.edge_values, [[1., 1.5, 0.5],
@@ -659,5 +660,5 @@
         #Create basic mesh
         points, vertices, boundary = rectangular(1, 3)
-
+        #print "vertices",vertices
         #Create shallow water domain
         domain = Domain(points, vertices, boundary)
@@ -678,9 +679,8 @@
         assert allclose(quantity.vertex_values[1], value[0])
 
-        # FIXMEDSG this seems wrong -DSG
-        # It's right.  The values is an array, indexed by the element id's
-        # given in indexes.  It sets values per triangle.
+
+        #print "quantity",quantity.vertex_values
         values = [10,100,50]
-        quantity.set_values(values, indexes = [0,1,5]) # , location = 'centroids'
+        quantity.set_values(values, indexes = [0,1,5], location = 'centroids')
         #print "2 quantity.vertex_values",quantity.vertex_values
         assert allclose(quantity.vertex_values[0], [10,10,10])
@@ -690,8 +690,22 @@
         assert allclose(quantity.centroid_values, [10,100,3,4,5,50])
 
+        
+        quantity = Quantity(domain,[[1,1,1],[2,2,2],[3,3,3],
+                                    [4,4,4],[5,5,5],[6,6,6]])
+        values = [10,100,50]
+        #this will be per unique vertex, indexing the vertices
+        #print "quantity.vertex_values",quantity.vertex_values
+        quantity.set_values(values, indexes = [0,1,5]) 
+        #print "quantity.vertex_values",quantity.vertex_values
+        assert allclose(quantity.vertex_values[0], [1,50,10])
+        assert allclose(quantity.vertex_values[5], [6,6,6])
+        assert allclose(quantity.vertex_values[1], [100,10,50])
+
+        quantity = Quantity(domain,[[1,1,1],[2,2,2],[3,3,3],
+                                    [4,4,4],[5,5,5],[6,6,6]])
         values = [[31,30,29],[400,400,400],[1000,999,998]]
         quantity.set_values(values, indexes = [3,3,5])
         quantity.interpolate()
-        assert allclose(quantity.centroid_values, [10,100,3,400,5,999])
+        assert allclose(quantity.centroid_values, [1,2,3,400,5,999])
 
         values = [[1,1,1],[2,2,2],[3,3,3],
@@ -711,4 +725,72 @@
                                                 [ 6.,  7.,  2.],
                                                 [ 3.,  2.,  7.]])
+    
+    def test_setting_unique_vertex_values(self):
+        """
+        set values based on unique_vertex lists.
+        """
+        from mesh_factory import rectangular
+        from shallow_water import Domain
+        from Numeric import zeros, Float
+        
+        #Create basic mesh
+        points, vertices, boundary = rectangular(1, 3)
+        #print "vertices",vertices
+        #Create shallow water domain
+        domain = Domain(points, vertices, boundary)
+        #print "domain.number_of_elements ",domain.number_of_elements 
+        quantity = Quantity(domain,[[0,0,0],[1,1,1],[2,2,2],[3,3,3],
+                                    [4,4,4],[5,5,5]])
+        value = 7
+        indexes = [1,5]
+        quantity.set_values(value,
+                                  location = 'unique vertices',
+                                  indexes = indexes)
+        #print "quantity.centroid_values",quantity.centroid_values
+        assert allclose(quantity.vertex_values[0], [0,7,0])
+        assert allclose(quantity.vertex_values[1], [7,1,7])
+        assert allclose(quantity.vertex_values[2], [7,2,7])
+        
+    
+    def test_get_values(self):
+        """
+        get values based on triangle lists.
+        """
+        from mesh_factory import rectangular
+        from shallow_water import Domain
+        from Numeric import zeros, Float
+        
+        #Create basic mesh
+        points, vertices, boundary = rectangular(1, 3)
+
+        #print "points",points
+        #print "vertices",vertices
+        #print "boundary",boundary
+
+        #Create shallow water domain
+        domain = Domain(points, vertices, boundary)
+        #print "domain.number_of_elements ",domain.number_of_elements 
+        quantity = Quantity(domain,[[0,0,0],[1,1,1],[2,2,2],[3,3,3],
+                                    [4,4,4],[5,5,5]])
+        
+        #print "quantity.get_values(location = 'unique vertices')", \
+        #      quantity.get_values(location = 'unique vertices')
+
+        #print "quantity.get_values(location = 'unique vertices')", \
+        #      quantity.get_values(indexes=[0,1,2,3,4,5,6,7], \
+        #                          location = 'unique vertices')
+        
+        answer = [0.5,2,4,5,0,1,3,4.5]
+        assert allclose(answer,
+                        quantity.get_values(location = 'unique vertices'))
+        
+        indexes = [0,5,3]
+        answer = [0.5,1,5]
+        assert allclose(answer,
+                        quantity.get_values(indexes=indexes, \
+                                            location = 'unique vertices'))
+        #print "quantity.centroid_values",quantity.centroid_values
+        #print "quantity.get_values(location = 'centroids') ",\
+        #      quantity.get_values(location = 'centroids')
         
     def test_getting_some_vertex_values(self):
@@ -775,4 +857,6 @@
 if __name__ == "__main__":
     suite = unittest.makeSuite(TestCase,'test')
+    #print "restricted test"
+    #suite = unittest.makeSuite(TestCase,'test_set_vertex_values_subset')
     runner = unittest.TextTestRunner()
     runner.run(suite)
Index: inundation/ga/storm_surge/pyvolution/test_region.py
===================================================================
--- inundation/ga/storm_surge/pyvolution/test_region.py	(revision 714)
+++ inundation/ga/storm_surge/pyvolution/test_region.py	(revision 715)
@@ -44,6 +44,6 @@
         domain.set_quantity('friction', manning)
 
-        a = Set_Region('bottom', 'friction', 0.09)
-        b = Set_Region('top', 'friction', 1.0)
+        a = Set_region('bottom', 'friction', 0.09)
+        b = Set_region('top', 'friction', 1.0)
         domain.set_region([a, b])
         #print domain.quantities['friction'].get_values()
@@ -56,6 +56,6 @@
                          [ 1.0,  1.0,  1.0]])
 
-        #c = Add_Value_To_Region('all', 'friction', 10.0)
-        domain.set_region(Add_Value_To_Region('all', 'friction', 10.0))
+        #c = Add_Value_To_region('all', 'friction', 10.0)
+        domain.set_region(Add_value_to_region('all', 'friction', 10.0))
         #print domain.quantities['friction'].get_values()
         assert allclose(domain.quantities['friction'].get_values(),
@@ -68,5 +68,5 @@
                          
         # trying a function                 
-        domain.set_region(Set_Region('top', 'friction', add_x_y))
+        domain.set_region(Set_region('top', 'friction', add_x_y))
         #print domain.quantities['friction'].get_values()
         assert allclose(domain.quantities['friction'].get_values(),
@@ -77,5 +77,81 @@
                          [ 5./3,  2.0,  2./3],
                          [ 1.0,  2./3,  2.0]])
-          
+                         
+        domain.set_quantity('elevation', 10.0)
+        domain.set_quantity('level', 10.0)
+        domain.set_region(Add_value_to_region('top', 'level', 1.0,initial_quantity='elevation'))
+        #print domain.quantities['level'].get_values()
+        assert allclose(domain.quantities['level'].get_values(),
+                        [[ 10., 10., 10.],
+                         [ 10., 10., 10.],
+                         [ 10., 10., 10.],
+                         [ 10., 10., 10.],
+                         [ 11.0,  11.0,  11.0],
+                         [ 11.0,  11.0,  11.0]])
+    
+    def test_unique_vertices(self):
+        """
+        get values based on triangle lists.
+        """
+        from mesh_factory import rectangular
+        from shallow_water import Domain
+        from Numeric import zeros, Float
+        
+        #Create basic mesh
+        points, vertices, boundary = rectangular(1, 3)
+        
+        #Create shallow water domain
+        domain = Domain(points, vertices, boundary)
+        domain.build_tagged_elements_dictionary({'bottom':[0,1],
+                                                 'top':[4,5],
+                                                 'all':[0,1,2,3,4,5]})
+      
+        #Set friction
+        manning = 0.07
+        domain.set_quantity('friction', manning)
+
+        a = Set_region('bottom', 'friction', 0.09, location = 'unique vertices')
+        domain.set_region(a)
+        #print domain.quantities['friction'].get_values()
+        assert allclose(domain.quantities['friction'].get_values(),\
+                        [[ 0.09,  0.09,  0.09],
+                         [ 0.09,  0.09,  0.09],
+                         [ 0.09,  0.07,  0.09],
+                         [ 0.07,  0.09,  0.07],
+                         [ 0.07,  0.07,  0.07],
+                         [ 0.07,  0.07,  0.07]])
+
+        
+    def test_unique_verticesII(self):
+        """
+        get values based on triangle lists.
+        """
+        from mesh_factory import rectangular
+        from shallow_water import Domain
+        from Numeric import zeros, Float
+        
+        #Create basic mesh
+        points, vertices, boundary = rectangular(1, 3)
+        
+        #Create shallow water domain
+        domain = Domain(points, vertices, boundary)
+        domain.build_tagged_elements_dictionary({'bottom':[0,1],
+                                                 'top':[4,5],
+                                                 'all':[0,1,2,3,4,5]})
+      
+        #Set friction
+        manning = 0.07
+        domain.set_quantity('friction', manning)
+
+        domain.set_region(Add_value_to_region('bottom', 'friction', 1.0,initial_quantity='friction', location = 'unique vertices'))
+        
+        #print domain.quantities['friction'].get_values()
+        assert allclose(domain.quantities['friction'].get_values(),\
+                        [[ 1.07,  1.07,  1.07],
+                         [ 1.07,  1.07,  1.07],
+                         [ 1.07,  0.07,  1.07],
+                         [ 0.07,  1.07,  0.07],
+                         [ 0.07,  0.07,  0.07],
+                         [ 0.07,  0.07,  0.07]])
 #-------------------------------------------------------------
 if __name__ == "__main__":

Index: inundation/ga/storm_surge/pyvolution/quantity.py
===================================================================
--- inundation/ga/storm_surge/pyvolution/quantity.py	(revision 714)
+++ inundation/ga/storm_surge/pyvolution/quantity.py	(revision 715)
@@ -94,6 +94,12 @@
         internal ordering.
 
-        If values are described a function, it will be evaluated at specified points
-        Indexes is the set of element ids that the operation applies to.
+        If values are described a function, it will be evaluated at
+        specified points
+        
+        If indexex is not 'unique vertices' Indexes is the set of element ids
+        that the operation applies to.
+        If indexex is 'unique vertices' Indexes is the set of vertex ids
+        that the operation applies to.
+        
         
         If selected location is vertices, values for centroid and edges
@@ -103,5 +109,5 @@
         """
 
-        if location not in ['vertices', 'centroids', 'edges']:
+        if location not in ['vertices', 'centroids', 'edges', 'unique vertices']:
             msg = 'Invalid location: %s' %location
             raise msg
@@ -136,4 +142,24 @@
                     for i in indexes:
                         self.edge_values[i,:] = X
+                        
+            elif location == 'unique vertices':
+                if (indexes ==  None):
+                    self.edge_values[:] = X
+                else:
+
+                    #Go through list of unique vertices
+                    for unique_vert_id in indexes:
+                        triangles = self.domain.vertexlist[unique_vert_id]
+                        
+                        #In case there are unused points
+                        if triangles is None: continue 
+
+                        #Go through all triangle, vertex pairs
+                        #and set corresponding vertex value
+                        for triangle_id, vertex_id in triangles:
+                            self.vertex_values[triangle_id, vertex_id] = X
+                
+                        #Intialise centroid and edge_values
+                        self.interpolate()
             else:
                 if (indexes ==  None):
@@ -148,5 +174,5 @@
             self.set_array_values(X, location, indexes = indexes)
 
-        if location == 'vertices':
+        if location == 'vertices' or location == 'unique vertices':
             #Intialise centroid and edge_values
             self.interpolate()
@@ -177,5 +203,5 @@
         from Numeric import take
 
-        if location not in ['vertices', 'centroids', 'edges']:
+        if location not in ['vertices', 'centroids', 'edges', 'unique vertices']:
             msg = 'Invalid location: %s' %location
             raise msg         
@@ -185,14 +211,36 @@
                                  Numeric.ArrayType],\
                                  'Indices must be a list or None'
-
-        if (indexes ==  None):
-            indexes = range(len(self))
-        
             
         if location == 'centroids':
+            if (indexes ==  None):
+                indexes = range(len(self))
             return take(self.centroid_values,indexes)                       
         elif location == 'edges':
-            return take(self.edge_values,indexes)
-        else:
+            if (indexes ==  None):
+                indexes = range(len(self))
+            return take(self.edge_values,indexes)                      
+        elif location == 'unique vertices':
+            if (indexes ==  None):
+                indexes=range(self.domain.coordinates.shape[0])
+            vert_values = []
+            #Go through list of unique vertices
+            for unique_vert_id in indexes:
+                triangles = self.domain.vertexlist[unique_vert_id]
+                        
+                #In case there are unused points
+                if triangles is None: 
+                    msg = 'Unique vertex not associated with triangles' 
+                    raise msg 
+
+                # Go through all triangle, vertex pairs
+                # Average the values
+                sum = 0
+                for triangle_id, vertex_id in triangles:
+                    sum += self.vertex_values[triangle_id, vertex_id]
+                vert_values.append(sum/len(triangles))
+            return Numeric.array(vert_values)
+        else:
+            if (indexes ==  None):
+                indexes = range(len(self))
             return take(self.vertex_values,indexes)
 
@@ -238,9 +286,10 @@
         values: Numeric array
         location: Where values are to be stored.
-                  Permissible options are: vertices, edges, centroid
-                  Default is "vertices"
-        
-        indexes - if this action is carried out on a subset of elements
-        The element indexes are specified here.
+        Permissible options are: vertices, edges, centroid, unique vertices
+        Default is "vertices"
+        
+        indexes - if this action is carried out on a subset of
+        elements or unique vertices
+        The element/unique vertex indexes are specified here.
         
         In case of location == 'centroid' the dimension values must
@@ -296,15 +345,19 @@
             
             self.edge_values = values
+            
+        elif location == 'unique vertices':
+            assert len(values.shape) == 1, 'Values array must be 1d'
+            self.set_vertex_values(values, indexes = indexes)
         else:
             if len(values.shape) == 1:
-
-                if indexes == None:
+                self.set_vertex_values(values, indexes = indexes)
+                #if indexes == None:
                     #Values are being specified once for each unique vertex
-                    msg = 'Number of values must match number of vertices'
-                    assert values.shape[0] == self.domain.coordinates.shape[0], msg
-                    self.set_vertex_values(values)
-                else:
-                    for element_index, value in map(None, indexes, values): 
-                        self.vertex_values[element_index, :] = value
+                #    msg = 'Number of values must match number of vertices'
+                #    assert values.shape[0] == self.domain.coordinates.shape[0], msg
+                 #   self.set_vertex_values(values)
+                #else:
+                #    for element_index, value in map(None, indexes, values): 
+                #        self.vertex_values[element_index, :] = value
                         
             elif len(values.shape) == 2:
@@ -325,9 +378,14 @@
 
     # FIXME have a get_vertex_values as well, so the 'level' quantity can be
-    # set, based on the elevation    
-    def set_vertex_values(self, A):
-        """Set vertex values for all triangles based on input array A
-        which is assumed to have one entry per (unique) vertex, i.e.
-        one value for each row in array self.domain.coordinates.
+    # set, based on the elevation
+    def set_vertex_values(self, A, indexes = None):
+        """Set vertex values for all unique vertices based on input array A
+        which has one entry per unique vertex, i.e.
+        one value for each row in array self.domain.coordinates or
+        one value for each row in vertexlist.
+
+        indexes is the list of vertex_id's that will be set. 
+
+        Note: Functions not allowed
         """
 
@@ -338,22 +396,24 @@
 
         assert len(A.shape) == 1
-        assert A.shape[0] == self.domain.coordinates.shape[0]
-
-        N = A.shape[0]
-        
+
+        if indexes == None:
+            assert A.shape[0] == self.domain.coordinates.shape[0]
+            vertex_list = range(A.shape[0])
+        else:
+            assert A.shape[0] == len(indexes)
+            vertex_list = indexes
         #Go through list of unique vertices
-        for k in range(N):
-            L = self.domain.vertexlist[k]
-                
-            if L is None: continue #In case there are unused points
+        for i_index,unique_vert_id in enumerate(vertex_list):
+            triangles = self.domain.vertexlist[unique_vert_id]
+                
+            if triangles is None: continue #In case there are unused points
 
             #Go through all triangle, vertex pairs
-            #touching vertex k and set corresponding vertex value
-            for triangle_id, vertex_id in L:
-                self.vertex_values[triangle_id, vertex_id] = A[k]
+            #touching vertex unique_vert_id and set corresponding vertex value
+            for triangle_id, vertex_id in triangles:
+                self.vertex_values[triangle_id, vertex_id] = A[i_index]
                 
         #Intialise centroid and edge_values
         self.interpolate()
-
  
     def smooth_vertex_values(self, value_array='field_values',